/**
 * Program: mqrc2
 *
 * Description:
 *  MQRC2 application will interpret an MQ reason code (i.e. 2035),
 *  MQ reason code symbol (i.e. MQRC_OBJECT_IN_USE) or AMQ message
 *  number (i.e. AMQ4005) and display the relevant information
 *  from the WebSphere MQ Messages manual.
 *
 *  The information contained in the 'ibm_amq.h' and 'ibm.rc.h' files
 *  was exported from the  WebSphere MQ Messages manual.  The
 * information belongs to IBM and Capitalware takes no responsibility
 *  for the content.
 *  Examples:
 *    mqrc2 ####     (integer number)
 *    mqrc2 0x####   (hex number)
 *    mqrc2 rc_symbol
 *    mqrc2 -s rc_symbol
 *    mqrc2 AMQ####
 *    mqrc2 -a AMQ####
 *
 * @copyright Copyright(C), 2011, Capitalware Inc.
 * @author Roger Lacroix, Capitalware Inc.
 * @return 0 for ok or -1 for error
 * @version 1.1.0
 * @license Apache 2 License
 */
#if defined(__MVS__)
#define MVS /* define our MVS indicator */
#define _XOPEN_SOURCE_EXTENDED 1
#endif

#define PROGRAM_NAME     "mqrc2"
#define PROGRAM_VERSION  "1.1.1"
#define AUTHOR_NAME      "Capitalware Inc."
static const char * EYECATCHER_EyeCatcher  = PROGRAM_NAME "  ";
static const char * EYECATCHER_Version     = PROGRAM_VERSION "  ";
static const char * EYECATCHER_Author      = AUTHOR_NAME "  ";
static const char * EYECATCHER_Datestamp   = __DATE__ ;
static const char * EYECATCHER_Timestamp   = __TIME__ ;
static const char * EYECATCHER_Filename    = __FILE__ ;

/*
 * Headers
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* mqrc2 headers */
#if defined(MVS) || defined(OS400)
#include "mqrc2def.h"
#include "ibmrc.h"
#include "ibmamq.h"
#else
#include "hdrs/mqrc2.h"
#include "hdrs/ibm_rc.h"
#include "hdrs/ibm_amq.h"
#endif

/**
 * Function Name
 *  Mainline
 *
 * Description
 *  This is the main entry point to the program..
 *
 * Input parameters
 *  N/A
 *
 * Output
 *  Output reason code information to the user's screen
 *
 * Return Value
 *  0
 */
int main(int argc, char **argv)
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int   userRC = -1;
   int   index = 0;
   int   i = 0;
   char  NumFlag = 'Y';
   char  HexFlag = 'N';
   #define TEMP_BUF_LEN 256
   char  tempBuf[TEMP_BUF_LEN+1];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   if ( (1 == argc) ||
        (argc >= 4) ||
        ( (2 == argc) &&
          ( (0 == strcmp(argv[1], "-h")) ||
            (0 == strcmp(argv[1], "-?"))  ) ) )
   {
      Usage(argv[0]);
   }
   /**
    * In the form of:
    *   mqrc AMQ####
    *   or
    *   mqrc -a AMQ####
    */
   else if ( ( (2 == argc) &&
               ( (0 == memcmp(argv[1], "AMQ", 3)) ||
                 (0 == memcmp(argv[1], "amq", 3)) )  ) ||
             ( (3 == argc) &&
               (0 == strcmp(argv[1], "-a")) &&
               ( (0 == memcmp(argv[2], "AMQ", 3)) ||
                 (0 == memcmp(argv[2], "amq", 3)) )  ) )
   {
      /* extract the number - skip AMQ */
      if (2 == argc)
         strncpy(tempBuf, &argv[1][3], TEMP_BUF_LEN-1);
      else
         strncpy(tempBuf, &argv[2][3], TEMP_BUF_LEN-1);
      tempBuf[TEMP_BUF_LEN] = '\0'; /* make sure it ends with a null */

      if ( (0 == memcmp(tempBuf, "7A", 2)) ||
           (0 == memcmp(tempBuf, "7B", 2)) ||
           (0 == memcmp(tempBuf, "7a", 2)) ||
           (0 == memcmp(tempBuf, "7b", 2))  )
      {
         /* convert hex string to integer */
         userRC = strtol(tempBuf, 0, 16);
         HexFlag = 'Y';
      }
      else
      {
         i=0;
         while ('\0' != tempBuf[i])
         {
            /* check if all chars are digits */
            if (!isdigit(tempBuf[i]))
               NumFlag = 'N';
            i++;
         }
         if ('Y' == NumFlag)
            userRC = atoi(tempBuf);   /* convert string to integer */
      }

      if ('Y' == NumFlag)
      {
         if ( (index = BinarySearch(AMQ_Messages,
                                    userRC,
                                    AMQ_Messages_Size )) != NOT_FOUND )
            DumpAMQInfoToTheScreen( userRC,  index, HexFlag);
         else
            printf("\nNo matching AMQ message number\n");
      }
      else
         Usage(argv[0]);
   }
   /**
    * In the form of:
    *   mqrc MQRC_NOT_AUTHORIZED
    *   or
    *   mqrc -s MQRC_NOT_AUTHORIZED
    */
   else if ( ( (2 == argc) &&
               (0 == memcmp(argv[1], "MQRC", 4)) ) ||
             ( (3 == argc) &&
               (0 == strcmp(argv[1], "-s")) &&
               (0 == memcmp(argv[2], "MQRC", 4)) ) )
   {
      if ( (index = StringSearch( Reason_Codes,
                                  (argc == 2 ? argv[1] : argv[2]),
                                  Reason_Codes_Size )) != NOT_FOUND )
         DumpRCInfoToTheScreen( Reason_Codes[index].Index,  index);
      else
         printf("\nNo matching return code\n");
   }
   /**
    * In the form of (hex number):
    *   mqrc 0x####
    */
   else if ( (2 == argc) &&
             ( (0 == memcmp(argv[1], "0x", 2)) ||
               (0 == memcmp(argv[1], "0X", 2))  ) )
   {
      /* extract the number - skip 0x */
      if (2 == argc)
         strncpy(tempBuf, &argv[1][2], TEMP_BUF_LEN-1);
      else
         strncpy(tempBuf, &argv[2][2], TEMP_BUF_LEN-1);
      tempBuf[TEMP_BUF_LEN] = '\0'; /* make sure it ends with a null */

      /* convert hex string to integer */
      userRC = strtol(tempBuf, 0, 16);
      if ( (index = BinarySearch( Reason_Codes,
                                  userRC,
                                  Reason_Codes_Size )) != NOT_FOUND )
         DumpRCInfoToTheScreen( userRC,  index);
      else
         printf("\nNo matching return code\n");
   }
   /**
    * In the form of:
    *   mqrc ####
    */
   else if (2 == argc)
   {
      i=0;
      while ('\0' != argv[1][i])
      {
         if (!isdigit(argv[1][i]))
            NumFlag = 'N';
         i++;
      }

      if ('Y' == NumFlag)
      {
         userRC = atoi(argv[1]); /* convert string to integer */
         if ( (index = BinarySearch(Reason_Codes,
                                    userRC,
                                    Reason_Codes_Size )) != NOT_FOUND )
            DumpRCInfoToTheScreen( userRC,  index);
         else
            printf("\nNo matching return code\n");
      }
      else
         Usage(argv[0]);
   }
   else
      Usage(argv[0]);

   return 0;
}

/**
 * Function Name
 *  Usage
 *
 * Description
 *  This function will output the help info to the screen.
 *
 * Input parameters
 *  char *str
 *
 * Output
 *  To the screen
 *
 * Return Value
 *  N/A
 *
 */
void Usage(char *pName)
{
   printf("%s v%s by %s\n", PROGRAM_NAME, PROGRAM_VERSION,AUTHOR_NAME);
   printf("(c) Copyright %s 2011. All rights reserved.\n",AUTHOR_NAME);

   printf("\nUsage:\n");
   printf("\t%s ####          (integer number)\n",
          pName);
   printf("\t%s 0x####        (hex number)\n",
          pName);
   printf("\t%s rc_symbol \n",
          pName);
   printf("\t%s -s rc_symbol \n",
          pName);
   printf("\t%s AMQ####\n",
          pName);
   printf("\t%s -a AMQ####\n",
          pName);

   printf("\n\twhere:\n");
   printf("\t\t#### is the MQ reason code i.e. 2085\n");
   printf("\t\t0x#### is the MQ reason code in hex i.e. 0x7f3\n");
   printf("\t\trc_symbol is the MQ reason code symbol "\
          "i.e. MQRC_NOT_AUTHORIZED\n");
   printf("\t\tAMQ#### is the MQ AMQ message number i.e. AMQ4001\n");

#if defined(MVS) || defined(OS400)
   exit(4);
#else
   exit(-1);
#endif
}

/**
 * Function Name
 *  DumpRCInfoToTheScreen
 *
 * Description
 *  This function will output the info to the screen.
 *
 * Input parameters
 *  int userRC
 *  int index
 *
 * Output
 *  To the screen
 *
 * Return Value
 *  N/A
 *
 */
void DumpRCInfoToTheScreen(int userRC, int index)
{
   printf("\n%d (X'%0.4X') %s\n",
          userRC,
          userRC,
          Reason_Codes[index].header);

   if ('\0' != Reason_Codes[index].explanation[0])
      printf("\nExplanation:\n%s\n", Reason_Codes[index].explanation);

   if ('\0' != Reason_Codes[index].G1.compCode[0])
      printf("\nCompletion Code:\n%s\n",
             Reason_Codes[index].G1.compCode);

   if ('\0' != Reason_Codes[index].G3.pgrResp[0])
      printf("\nProgrammer Response:\n%s\n",
             Reason_Codes[index].G3.pgrResp);
}

/**
 * Function Name
 *  DumpAMQInfoToTheScreen
 *
 * Description
 *  This function will output the info to the screen.
 *
 * Input parameters
 *  int userRC
 *  int index
 *
 * Output
 *  To the screen
 *
 * Return Value
 *  N/A
 *
 */
void DumpAMQInfoToTheScreen(int userRC, int index, char HexFlag)
{
   if ('Y' == HexFlag)
      printf("\nAMQ%0.4X %s\n", userRC, AMQ_Messages[index].header);
   else
      printf("\nAMQ%d %s\n", userRC, AMQ_Messages[index].header);

   if ('\0' != AMQ_Messages[index].G1.severity[0])
      printf("\nSeverity:\n%s\n", AMQ_Messages[index].G1.severity);

   if ('\0' != AMQ_Messages[index].explanation[0])
      printf("\nExplanation:\n%s\n", AMQ_Messages[index].explanation);

   if ('\0' != AMQ_Messages[index].G3.response[0])
      printf("\nResponse:\n%s\n", AMQ_Messages[index].G3.response);
}

/**
 * Function Name
 *  BinarySearch
 *
 * Description
 *  This function will perform a binary search and return an
 *  integer to the matching item.
 *
 * Input parameters
 *  MQ_ARRAY_T A[ ]
 *  int searchItem
 *  int N
 *
 * Output
 *  N/A
 *
 * Return Value
 *  index or NOT_FOUND
 *
 */
int BinarySearch( MQ_ARRAY_T A[ ], int searchItem, int N )
{
   int Low = 0;
   int Mid;
   int High = N - 1;

   while( Low <= High )
   {
      Mid = ( Low + High ) / 2;
      if( A[ Mid ].Index < searchItem )
         Low = Mid + 1;
      else
      if( A[ Mid ].Index > searchItem )
         High = Mid - 1;
      else
         return Mid;  /* Found */
   }
   return NOT_FOUND;
}

/**
 * Function Name
 *  StringSearch
 *
 * Description
 *  This function will perform a search and return an
 *  integer to the matching item.
 *
 * Input parameters
 *  MQ_ARRAY_T A[ ]
 *  int searchItem
 *  int N
 *
 * Output
 *  N/A
 *
 * Return Value
 *  index or NOT_FOUND
 *
 */
int StringSearch( MQ_ARRAY_T A[ ], char *searchItem, int N )
{
   int i;

   for( i=0; i < N; i++ )
   {
       if (0 == strcmp( A[i].header, searchItem ))
          return i;
   }
   return NOT_FOUND;
}

/* End of file */
